import 'dart:ffi';
import 'dart:typed_data';
import 'package:ffi/ffi.dart' as ffi;

/// Stores the [ptr] to an array and its [length]
class Array<T extends NativeType> {
  final Pointer<T> ptr;
  final int length;
  final Allocator _allocator;

  Array.fromPtr(this.ptr, this.length, [this._allocator = ffi.malloc]);

  /// Frees [ptr] with [_allocator].
  void free() {
    _allocator.free(ptr);
  }
}

extension Uint8Array on Array<Uint8> {
  Uint8List asTypedList() => ptr.asTypedList(length);
  static Array<Uint8> allocate(int length,
          [Allocator allocator = ffi.malloc]) =>
      Array.fromPtr(allocator(length), length, allocator);
  static Array<Uint8> fromIterable(Iterable<int> list,
          [Allocator allocator = ffi.malloc]) =>
      allocate(list.length, allocator)..asTypedList().insertAll(0, list);
}
