import 'dart:ffi';

import 'package:ffi_helper/ffi_helper.dart';
import 'package:meta/meta.dart';
import 'delegating_array.dart';
import 'shared_array.dart';

/// References a [SharedArray] without influencing the reference count.
@sealed
class WeakArray<T extends NativeType> extends DelegatingArray<T> {
  final SharedArray<T> _shared;
  bool get deleted => _shared.deleted;
  @override
  //ignore:invalid_override_of_non_virtual_member
  Pointer<T> get ptr {
    assert(!deleted);
    return super.ptr;
  }

  WeakArray(this._shared) : super(_shared);

  /// Converts [this] to a [SharedArray]
  SharedArray<T> toShared() => _shared.newShared();

  /// Doesn't do anything
  @override
  void free() {}
}
